@@ -7,7 +7,6 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/canonical/ubuntu-pro-for-wsl/common/golden"
 	commontestutils "github.com/canonical/ubuntu-pro-for-wsl/common/testutils"
 	"github.com/canonical/ubuntu-pro-for-wsl/wsl-pro-service/internal/system"
 	"github.com/canonical/ubuntu-pro-for-wsl/wsl-pro-service/internal/testutils"
@@ -28,11 +27,7 @@ func TestInfo(t *testing.T) {
 
 	testCases := map[string]struct {
 		// This causes Get to look at the Windows' path for /
-		distroNameEnvDisabled bool
-
-		// The path of "/" according to the Windows host
-		distroNameWslPath mockBehaviour
-
+		badWslDistroName bool
 		proStatusCommand mockBehaviour
 		osRelease        mockBehaviour
 
@@ -40,11 +35,9 @@ func TestInfo(t *testing.T) {
 
 		wantErr bool
 	}{
-		"Success reading from WSL_DISTRO_NAME": {},
-		"Success using wslpath":                {distroNameEnvDisabled: true},
+		"Success": {},
 
-		"Error when WSL_DISTRO_NAME is empty and wslpath fails":            {distroNameEnvDisabled: true, distroNameWslPath: mockError, wantErr: true},
-		"Error when WSL_DISTRO_NAME is empty and wslpath returns bad text": {distroNameEnvDisabled: true, distroNameWslPath: mockBadOutput, wantErr: true},
+		"Error when WslDistroName fails": {badWslDistroName: true, wantErr: true},
 
 		"Error when pro status command fails":           {proStatusCommand: mockError, wantErr: true},
 		"Error when pro status output cannot be parsed": {proStatusCommand: mockBadOutput, wantErr: true},
@@ -56,7 +49,6 @@ func TestInfo(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 			ctx := context.Background()
@@ -64,7 +56,8 @@ func TestInfo(t *testing.T) {
 			system, mock := testutils.MockSystem(t)
 			mock.SetControlArg(testutils.ProStatusAttached)
 
-			if tc.distroNameEnvDisabled {
+			if tc.badWslDistroName {
+				mock.SetControlArg(testutils.WslpathErr)
 				mock.WslDistroNameEnvEnabled = false
 			}
 
@@ -72,16 +65,6 @@ func TestInfo(t *testing.T) {
 				mock.DistroHostname = nil
 			}
 
-			switch tc.distroNameWslPath {
-			case mockOK:
-			case mockError:
-				mock.SetControlArg(testutils.WslpathErr)
-			case mockBadOutput:
-				mock.SetControlArg(testutils.WslpathBadOutput)
-			default:
-				require.Fail(t, "Unknown enum value for distroNameWslPath", "Value: %d", tc.distroNameWslPath)
-			}
-
 			switch tc.proStatusCommand {
 			case mockOK:
 			case mockError:
@@ -105,10 +88,10 @@ func TestInfo(t *testing.T) {
 
 			info, err := system.Info(ctx)
 			if tc.wantErr {
-				require.Error(t, err, "Expected Get() to return an error")
+				require.Error(t, err, "Expected Info() to return an error")
 				return
 			}
-			require.NoError(t, err, "Expected Get() to return no errors")
+			require.NoError(t, err, "Expected Info() to return no errors")
 
 			assert.Equal(t, "TEST_DISTRO", info.GetWslName(), "WslName does not match expected value")
 			assert.Equal(t, "ubuntu", info.GetId(), "Id does not match expected value")
@@ -120,6 +103,63 @@ func TestInfo(t *testing.T) {
 	}
 }
 
+func TestWslDistroName(t *testing.T) {
+	t.Parallel()
+
+	testCases := map[string]struct {
+		// This causes Get to look at the Windows' path for /
+		distroNameEnvDisabled bool
+
+		// The path of "/" according to the Windows host
+		distroNameWslPath mockBehaviour
+
+		wantErr bool
+	}{
+		"Success reading from WSL_DISTRO_NAME": {},
+		"Success using wslpath":                {distroNameEnvDisabled: true},
+
+		"Error when WSL_DISTRO_NAME is empty and wslpath fails":            {distroNameEnvDisabled: true, distroNameWslPath: mockError, wantErr: true},
+		"Error when WSL_DISTRO_NAME is empty and wslpath returns bad text": {distroNameEnvDisabled: true, distroNameWslPath: mockBadOutput, wantErr: true},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			t.Parallel()
+			ctx := context.Background()
+
+			system, mock := testutils.MockSystem(t)
+
+			if tc.distroNameEnvDisabled {
+				mock.WslDistroNameEnvEnabled = false
+			}
+
+			switch tc.distroNameWslPath {
+			case mockOK:
+			case mockError:
+				mock.SetControlArg(testutils.WslpathErr)
+			case mockBadOutput:
+				mock.SetControlArg(testutils.WslpathBadOutput)
+			default:
+				require.Fail(t, "Unknown enum value for distroNameWslPath", "Value: %d", tc.distroNameWslPath)
+			}
+
+			got, err := system.WslDistroName(ctx)
+			if tc.wantErr {
+				require.Error(t, err, "Expected WslDistroName() to return an error")
+				return
+			}
+			require.NoError(t, err, "Expected WslDistroName() to return no errors")
+			assert.Equal(t, "TEST_DISTRO", got, "WslDistroName does not match expected value")
+
+			// Test the cache: second call should not call wslpath
+			mock.SetControlArg(testutils.WslpathErr)
+			got, err = system.WslDistroName(ctx)
+			require.NoError(t, err, "WslDistroName should return no error as the cache should be used")
+			assert.Equal(t, "TEST_DISTRO", got, "WslDistroName does not match expected value in second call")
+		})
+	}
+}
+
 func TestUserProfileDir(t *testing.T) {
 	t.Parallel()
 
@@ -148,7 +188,6 @@ func TestUserProfileDir(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
@@ -195,7 +234,7 @@ func TestUserProfileDir(t *testing.T) {
 func overrideProcMount(t *testing.T, mock *testutils.SystemMock) {
 	t.Helper()
 
-	procMount := filepath.Join(golden.TestFixturePath(t), "proc/mounts")
+	procMount := filepath.Join(commontestutils.TestFixturePath(t), "proc/mounts")
 	if _, err := os.Stat(procMount); err != nil {
 		require.ErrorIsf(t, err, os.ErrNotExist, "Setup: could not stat %q", procMount)
 
@@ -231,7 +270,6 @@ func TestProStatus(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
@@ -275,7 +313,6 @@ func TestProAttach(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
@@ -320,7 +357,6 @@ func TestProDetach(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
@@ -357,21 +393,24 @@ func TestLandscapeEnable(t *testing.T) {
 		breakWriteConfig     bool
 		breakLandscapeConfig bool
 		breakWSLPath         bool
+		noLandscapeGroup     bool
 
 		wantErr bool
 	}{
-		"Success":                                    {},
-		"Success overriding computer_title":          {},
-		"Success overriding the SSL certficate path": {},
+		"Success":                                                     {},
+		"Success overriding computer_title":                           {},
+		"Success overriding the SSL certficate path":                  {},
+		"Do not append wsl tag when config tag is provided":           {},
+		"Do not append wsl tag when config tag is provided but empty": {},
 
 		"Error when the file cannot be parsed":                   {wantErr: true},
 		"Error when the config file cannot be written":           {breakWriteConfig: true, wantErr: true},
 		"Error when the landscape-config command fails":          {breakLandscapeConfig: true, wantErr: true},
 		"Error when failing to override the SSL certficate path": {breakWSLPath: true, wantErr: true},
+		"Error when the Landscape user does not exist":           {noLandscapeGroup: true, wantErr: true},
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
@@ -391,7 +430,11 @@ func TestLandscapeEnable(t *testing.T) {
 				mock.SetControlArg(testutils.WslpathErr)
 			}
 
-			config, err := os.ReadFile(filepath.Join(golden.TestFixturePath(t), "landscape.conf"))
+			if tc.noLandscapeGroup {
+				mock.LandscapeGroupGID = ""
+			}
+
+			config, err := os.ReadFile(filepath.Join(commontestutils.TestFixturePath(t), "landscape.conf"))
 			require.NoError(t, err, "Setup: could not load fixture")
 
 			err = s.LandscapeEnable(ctx, string(config), "landscapeUID1234")
@@ -410,7 +453,7 @@ func TestLandscapeEnable(t *testing.T) {
 			// runs, so the golden file would never match. This is the solution:
 			got := strings.ReplaceAll(string(out), mock.FsRoot, "${FILESYSTEM_ROOT}")
 
-			want := golden.LoadWithUpdateFromGolden(t, got)
+			want := commontestutils.LoadWithUpdateFromGolden(t, got)
 			require.Equal(t, want, got, "Landscape executable did not receive the right config")
 		})
 	}
@@ -491,7 +534,6 @@ func TestWindowsHostAddress(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 			ctx := context.Background()
@@ -505,8 +547,8 @@ func TestWindowsHostAddress(t *testing.T) {
 				mock.SetControlArg(testutils.WslInfoIsNAT)
 			}
 
-			copyFile(t, tc.etcResolv, filepath.Join(golden.TestFamilyPath(t), "etc-resolv.conf"), mock.Path("/etc/resolv.conf"))
-			copyFile(t, tc.procNetRoute, filepath.Join(golden.TestFamilyPath(t), "proc-net-route"), mock.Path("/proc/net/route"))
+			copyFile(t, tc.etcResolv, filepath.Join(commontestutils.TestFamilyPath(t), "etc-resolv.conf"), mock.Path("/etc/resolv.conf"))
+			copyFile(t, tc.procNetRoute, filepath.Join(commontestutils.TestFamilyPath(t), "proc-net-route"), mock.Path("/proc/net/route"))
 
 			got, err := sys.WindowsHostAddress(ctx)
 			if tc.wantErr {
@@ -533,7 +575,6 @@ func TestLandscapeDisable(t *testing.T) {
 	}
 
 	for name, tc := range testCases {
-		tc := tc
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
@@ -556,6 +597,46 @@ func TestLandscapeDisable(t *testing.T) {
 	}
 }
 
+func TestRealBackend(t *testing.T) {
+	t.Parallel()
+
+	ctx := context.Background()
+	b := system.RealBackend{}
+
+	// Asserting generated commands
+	//
+	// Note that we cannot test the cmd.Path directly as it will depend on the install location,
+	// so we only test the base of the path.
+
+	pro := b.ProExecutable(ctx, "arg1", "arg2")
+	assertBasePath(t, "pro", pro.Path, "ProExecutable did not return the expected command")
+	assert.Equal(t, []string{"pro", "arg1", "arg2"}, pro.Args, "ProExecutable did not return the expected arguments")
+
+	lpe := b.LandscapeConfigExecutable(ctx, "arg1", "arg2")
+	assertBasePath(t, "landscape-config", lpe.Path, "LandscapeConfigExecutable did not return the expected command")
+	assert.Equal(t, []string{"landscape-config", "arg1", "arg2"}, lpe.Args, "LandscapeConfigExecutable did not return the expected arguments")
+
+	wpath := b.WslpathExecutable(ctx, "arg1", "arg2")
+	assertBasePath(t, "wslpath", wpath.Path, "WslpathExecutable did not return the expected command")
+	assert.Equal(t, []string{"wslpath", "arg1", "arg2"}, wpath.Args, "WslpathExecutable did not return the expected arguments")
+
+	winfo := b.WslinfoExecutable(ctx, "arg1", "arg2")
+	assertBasePath(t, "wslinfo", winfo.Path, "WslinfoExecutable did not return the expected command")
+	assert.Equal(t, []string{"wslinfo", "arg1", "arg2"}, winfo.Args, "WslinfoExecutable did not return the expected arguments")
+
+	cmd := b.CmdExe(ctx, "/mnt/c/WINDOWS/whatever/cmd.exe", "arg1", "arg2")
+	assert.Equal(t, "/mnt/c/WINDOWS/whatever", cmd.Dir, "CmdExe did not set the expected directory")
+	assert.Equal(t, "/mnt/c/WINDOWS/whatever/cmd.exe", cmd.Path, "CmdExe did not return the expected command")
+	assert.Equal(t, []string{"/mnt/c/WINDOWS/whatever/cmd.exe", "arg1", "arg2"}, cmd.Args, "CmdExe did not return the expected arguments")
+}
+
+// Asserts that the base of got is equal to wantBase, and if not, it fails the test with a message.
+func assertBasePath(t *testing.T, wantBase, got, msg string) {
+	t.Helper()
+	base := filepath.Base(got)
+	assert.Equalf(t, wantBase, base, "Mismatch in base path.\n%s", msg)
+}
+
 func TestWithProMock(t *testing.T)             { testutils.ProMock(t) }
 func TestWithLandscapeConfigMock(t *testing.T) { testutils.LandscapeConfigMock(t) }
 func TestWithWslPathMock(t *testing.T)         { testutils.WslPathMock(t) }
