@@ -3,10 +3,13 @@
 package testutils
 
 import (
+	"context"
 	_ "embed"
 	"errors"
 	"fmt"
 	"os"
+	"os/exec"
+	"os/user"
 	"path/filepath"
 	"strings"
 	"syscall"
@@ -34,6 +37,9 @@ type SystemMock struct {
 	// string when false
 	WslDistroNameEnvEnabled bool
 
+	// LookupGroupError makes the LookupGroup function fail.
+	LandscapeGroupGID string
+
 	// extraEnv are extra environment variables that will be passed to mocked executables
 	extraEnv []string
 }
@@ -46,8 +52,8 @@ var (
 	windowsUserProfileDir = `D:\Users\TestUser\`
 	linuxUserProfileDir   = "/mnt/d/Users/TestUser/"
 
-	// defaultAddrFile is the default path used in tests to store the address of the Windows Agent service.
-	defaultAddrFile = filepath.Join(linuxUserProfileDir, common.UserProfileDir, common.ListeningPortFileName)
+	// defaultPublicDir is the default path used in tests to store the address of the Windows Agent service.
+	defaultPublicDir = filepath.Join(linuxUserProfileDir, common.UserProfileDir)
 
 	//go:embed filesystem_defaults/os-release
 	defaultOsReleaseContents []byte
@@ -109,24 +115,28 @@ const (
 
 // MockSystem sets up a few mocks:
 // - filesystem and mock executables for wslpath, pro.
-func MockSystem(t *testing.T) (system.System, *SystemMock) {
+func MockSystem(t *testing.T) (*system.System, *SystemMock) {
 	t.Helper()
 
+	u, err := user.Current()
+	require.NoError(t, err, "Setup: could not get current user")
+
 	distroHostname := "TEST_DISTRO_HOSTNAME"
 	mock := &SystemMock{
 		FsRoot:                  mockFilesystemRoot(t),
 		WslDistroName:           "TEST_DISTRO",
 		DistroHostname:          &distroHostname,
 		WslDistroNameEnvEnabled: true,
+		LandscapeGroupGID:       u.Gid,
 	}
 
 	return system.New(system.WithTestBackend(mock)), mock
 }
 
-// DefaultAddrFile is the location where a mocked system will expect the addr file to be located,
+// DefaultPublicDir is the location where a mocked system will expect the addr file to be located,
 // and its containing directory will be created in New().
-func (m *SystemMock) DefaultAddrFile() string {
-	return m.Path(defaultAddrFile)
+func (m *SystemMock) DefaultPublicDir() string {
+	return m.Path(defaultPublicDir)
 }
 
 // SetControlArg adds control arguments to the mock executables.
@@ -156,76 +166,94 @@ func (m *SystemMock) GetenvWslDistroName() string {
 	return ""
 }
 
+// LookupGroup mocks the user.LookupGroup function.
+func (m *SystemMock) LookupGroup(name string) (*user.Group, error) {
+	if name != "landscape" {
+		return nil, fmt.Errorf("mock does not support group %q", name)
+	}
+
+	if m.LandscapeGroupGID == "" {
+		return nil, user.UnknownGroupError(name)
+	}
+
+	return &user.Group{
+		Gid:  m.LandscapeGroupGID,
+		Name: name,
+	}, nil
+}
+
 // mockExec generates a command of the form `bash -ec <SCRIPT>` that will call an alternate binary
 // to the one we are mocking.
 //
 // At the core of the script we have
 //
 //	```
-//	SWITCH1=1 SWITCH1=2 go test -run <FAUX_TEST> -- <ARGS...>
+//	go test -run ^<FAUX_TEST>$ -- <ARGS...>
 //	````
 //
-// The switches control the behaviour of the mock, and FAUX_TEST is the name of a Test* function
+// The environment controls the behaviour of the mock, and FAUX_TEST is the name of a Test* function
 // that mocks the behaviour of the executable. The ARGS are the arguments that would be passed to
 // the real binary, in this case being passed to the mocked one.
 //
-// The faux test is in charge of interpreting the switches and the args.
+// The faux test is in charge of interpreting the environment and the args.
 //
 // The script has some more boilerplate to trim out text from the testing module.
 // In order to make the mock work, the faux test needs to be defined in the test module,
 // see the documentation on ProMock for an example.
-func (m *SystemMock) mockExec(fauxTestName string, argv ...string) (string, []string) {
+func (m *SystemMock) mockExec(ctx context.Context, fauxTestName string, argv ...string) *exec.Cmd {
+	if !testing.Testing() {
+		panic("mockExec can only be used within a test")
+	}
+
 	// Switches
-	env := make([]string, len(m.extraEnv))
-	copy(env, m.extraEnv)
+	env := append(os.Environ(), m.extraEnv...)
 	env = append(env,
 		fmt.Sprintf("%s=1", mockExecutable),                      // Ensures the faux test is not skipped
-		fmt.Sprintf("%s=%q", wslpathDistroName, m.WslDistroName), // Informs the faux tests what the mock distro name is
-		fmt.Sprintf("%s=%q", FileSystemRoot, m.FsRoot),           // Indicates where the mock filesystem is
+		fmt.Sprintf("%s=%s", wslpathDistroName, m.WslDistroName), // Informs the faux tests what the mock distro name is
+		fmt.Sprintf("%s=%s", FileSystemRoot, m.FsRoot),           // Indicates where the mock filesystem is
 	)
-	switches := strings.Join(env, " ")
 
-	// Supplanted executable
-	exec := fmt.Sprintf("go test -run ^%s$", fauxTestName)
-
-	// Arguments
+	// Quote arguments
 	for i := range argv {
 		argv[i] = fmt.Sprintf("%q", argv[i])
 	}
 	args := strings.Join(argv, " ")
 
 	// Heart of the script
-	script := fmt.Sprintf("%s %s -- %s", switches, exec, args)
+	heart := fmt.Sprintf("go test -run ^%s$ -- %s", fauxTestName, args)
 
 	// Trimming testing framework text
-	script = fmt.Sprintf("set -o pipefail && %s | head -n -2", script)
+	script := fmt.Sprintf("set -o pipefail && %s | head -n -2", heart)
 
-	return "bash", []string{"-ec", script}
+	//nolint:gosec // This is test code
+	cmd := exec.CommandContext(ctx, "bash", "-ec", script)
+	cmd.Env = env
+	return cmd
 }
 
 // ProExecutable mocks `pro $args...`.
-func (m *SystemMock) ProExecutable(args ...string) (string, []string) {
-	return m.mockExec("TestWithProMock", args...)
+func (m *SystemMock) ProExecutable(ctx context.Context, args ...string) *exec.Cmd {
+	return m.mockExec(ctx, "TestWithProMock", args...)
 }
 
 // LandscapeConfigExecutable mocks `landscape-config $q`.
-func (m *SystemMock) LandscapeConfigExecutable(args ...string) (string, []string) {
-	return m.mockExec("TestWithLandscapeConfigMock", args...)
+func (m *SystemMock) LandscapeConfigExecutable(ctx context.Context, args ...string) *exec.Cmd {
+	return m.mockExec(ctx, "TestWithLandscapeConfigMock", args...)
 }
 
 // WslpathExecutable mocks `wslpath $args...`.
-func (m *SystemMock) WslpathExecutable(args ...string) (string, []string) {
-	return m.mockExec("TestWithWslPathMock", args...)
+func (m *SystemMock) WslpathExecutable(ctx context.Context, args ...string) *exec.Cmd {
+	return m.mockExec(ctx, "TestWithWslPathMock", args...)
 }
 
 // WslinfoExecutable mocks `wslinfo $args...`.
-func (m *SystemMock) WslinfoExecutable(args ...string) (string, []string) {
-	return m.mockExec("TestWithWslInfoMock", args...)
+func (m *SystemMock) WslinfoExecutable(ctx context.Context, args ...string) *exec.Cmd {
+	return m.mockExec(ctx, "TestWithWslInfoMock", args...)
 }
 
 // CmdExe mocks `cmd.exe $args...`.
-func (m *SystemMock) CmdExe(path string, args ...string) (string, []string) {
-	return m.mockExec("TestWithCmdExeMock", args...)
+func (m *SystemMock) CmdExe(ctx context.Context, path string, args ...string) *exec.Cmd {
+	return m.mockExec(ctx, "TestWithCmdExeMock", args...)
 }
 
 type exitCode int
@@ -272,6 +300,19 @@ func ProMock(t *testing.T) {
 				fmt.Fprintln(os.Stdout, `{"message": "This error is produced by a mock instructed to fail on pro attach", "message_code": "mock_error"}`)
 				return exitError
 			}
+
+			// Proving that this executable has run
+			root := os.Getenv(FileSystemRoot)
+			if root == "" {
+				fmt.Fprintf(os.Stderr, "Missing environment variable %s\n", FileSystemRoot)
+				return exitBadUsage
+			}
+
+			p := filepath.Join(root, ".pro-attached")
+			if err := os.WriteFile(p, []byte{}, 0600); err != nil {
+				fmt.Fprintf(os.Stderr, "Error: could not write file: %v", err)
+			}
+
 			return exitOk
 
 		case "detach":
@@ -298,6 +339,18 @@ func ProMock(t *testing.T) {
 				return exitError
 			}
 
+			// Proving that this executable has run
+			root := os.Getenv(FileSystemRoot)
+			if root == "" {
+				fmt.Fprintf(os.Stderr, "Missing environment variable %s\n", FileSystemRoot)
+				return exitBadUsage
+			}
+
+			p := filepath.Join(root, ".pro-detached")
+			if err := os.WriteFile(p, []byte{}, 0600); err != nil {
+				fmt.Fprintf(os.Stderr, "Error: could not write file: %v", err)
+			}
+
 			return exitOk
 		default:
 			fmt.Fprintf(os.Stderr, "Unknown verb %q", argv[0])
@@ -622,9 +675,9 @@ func mockFilesystemRoot(t *testing.T) (rootDir string) {
 	require.NoError(t, err, "Setup: could not write mock /proc/mounts")
 
 	// Mock Windows FS
-	portDir := filepath.Join(rootDir, defaultAddrFile)
-	err = os.MkdirAll(filepath.Dir(portDir), 0750)
-	require.NoErrorf(t, err, "Setup: could not create mock %s", portDir)
+	publicDir := filepath.Join(rootDir, defaultPublicDir)
+	err = os.MkdirAll(publicDir, 0750)
+	require.NoErrorf(t, err, "Setup: could not create mock %s", publicDir)
 
 	system32 := filepath.Join(rootDir, defaultWindowsMount, "WINDOWS/system32")
 	err = os.MkdirAll(system32, 0750)
